import * as vue_demi from 'vue-demi';
import { Ref, UnwrapRef, UnwrapNestedRefs as UnwrapNestedRefs$1, ComputedRef, InjectionKey, App as App$1 } from 'vue-demi';
import { AnyFunction, DropParameters } from '@winter-love/utils';
import { CustomInspectorNode, StateBase, App } from '@vue/devtools-api';

declare type OneAndAnyFunc<T, A extends any[] = any[], R = any> = (one: T, ...args: A) => R;
declare type UnwrapNestedRefs<T> = T extends Ref ? T : UnwrapRef<T>;
declare type DropFunctionObject<T extends Record<string, AnyFunction>, S = any> = {
    [P in keyof T]: (...args: DropParameters<T[P], S>) => ReturnType<T[P]>;
};

interface StoreManagerItem<T extends Record<string, any> = Record<string, any>> {
    state: UnwrapNestedRefs$1<T>;
}
interface StoreTreeInfo {
    readonly kind: string;
}
interface StoreTree {
    [key: string]: any;
}
declare const useStoreManager: () => Readonly<{
    get(name: string): any;
    readonly initState: Readonly<Record<string, any>>;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem): void;
    setInitState: (state?: Record<string, any> | undefined) => void;
    state: ComputedRef<Record<string, any>>;
    readonly store: ManagerData;
}>;
declare type StoreManager = Readonly<{
    get(name: string): any;
    readonly initState: Readonly<Record<string, any>>;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem): void;
    setInitState: (state?: Record<string, any>) => void;
    state: ComputedRef<Record<string, any>>;
    readonly store: ManagerData;
}>;
interface ManagerData {
    readonly info?: StoreTreeInfo;
    readonly tree: UnwrapNestedRefs$1<StoreTree>;
}
declare type Plugin = (state: ComputedRef<Record<string, any>>, info?: StoreTreeInfo) => unknown;
declare const createManager: (info?: StoreTreeInfo | undefined) => StoreManager;

declare const createTree: (store: ManagerData, index?: number) => CustomInspectorNode[];
declare const createTreeMultiple: (...stores: ManagerData[]) => CustomInspectorNode[];
declare const createInspect: (name: any, state: UnwrapNestedRefs$1<any>) => Record<string, StateBase[]>;
declare const createStoreDevTool: (app: App, stores: ManagerData | ManagerData[]) => void;

declare const STORE_CONTEXT: InjectionKey<StoreManager>;
declare const STORE_LOCAL_CONTEXT: InjectionKey<StoreManager>;

declare type EmptyObject = {};
interface UseStore<T> {
    readonly local: (options?: Omit<UseStoreOptions, 'local'>) => Store<T>;
    (options?: UseStoreOptions): Store<T>;
}
declare type ObjectWithRef<T extends Record<string, any>> = {
    [P in keyof T]: Ref<T[P]> | T[P];
};
declare type Store<T> = UnwrapNestedRefs$1<T> & {
    readonly __store?: never;
};
interface UseStoreOptions {
    /**
     * use as a normal composition api function
     */
    local?: boolean;
    /**
     * reset saved store
     */
    reset?: boolean;
}
interface SetupContext<Root> {
    readonly root: UnwrapNestedRefs$1<Root>;
}
declare type Setup<T extends Record<string, any>, Root extends Record<string, any> = Record<string, any>> = (initState: Partial<T>, root: UnwrapNestedRefs$1<Root>) => T;
interface CreateStoreOptions<T extends Record<string, any>> {
    /**
     * @experimental use at using time
     */
    local?: boolean;
    name: string;
    setup: Setup<T>;
}
interface CreateSetupArgs<T extends Record<string, any>> {
    initState: Partial<T>;
    setup: Setup<T>;
    storeManager: StoreManager;
}
interface CreateLocalStoreArgs<T extends Record<string, any>> extends Omit<CreateSetupArgs<T>, 'initState'> {
    name: string;
}
interface CreateGlobalStoreArgs<T extends Record<string, any>> {
    name: string;
    options: Omit<UseStoreOptions, 'local'>;
    setup: Setup<T>;
    storeManager: StoreManager;
}
declare const useLocalManager: () => StoreManager | undefined;
declare function defineStore<T extends Record<string, any>>(options: CreateStoreOptions<T>): UseStore<T>;
declare function defineStore<T extends Record<string, any>>(name: string, setup: Setup<T, EmptyObject>): UseStore<T>;
declare function createStore<T extends Record<string, any>>(options: CreateStoreOptions<T>): UseStore<T>;
declare function createStore<T extends Record<string, any>>(name: string, setup: Setup<T, EmptyObject>): UseStore<T>;
/**
 * @deprecated
 * @param options
 */
declare function useStore<T extends Record<string, any>>(options: CreateStoreOptions<T>): any;
declare function useStore<T extends Record<string, any>>(name: string, setup: Setup<T>): any;

interface PluginOptions {
    initState?: Record<string, any>;
    plugins?: Plugin[];
}
interface Vare {
    install: (app: App$1, options?: PluginOptions) => void;
    localManager: StoreManager;
    manager: StoreManager;
}
declare const createVare: () => Vare;
declare const createVareStore: () => Vare;
declare const provideStoreManager: (manager?: Readonly<{
    get(name: string): any;
    readonly initState: Readonly<Record<string, any>>;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem<Record<string, any>>): void;
    setInitState: (state?: Record<string, any> | undefined) => void;
    state: vue_demi.ComputedRef<Record<string, any>>;
    readonly store: ManagerData;
}> | undefined, localManager?: Readonly<{
    get(name: string): any;
    readonly initState: Readonly<Record<string, any>>;
    remove(name: string): void;
    set(name: string, item: StoreManagerItem<Record<string, any>>): void;
    setInitState: (state?: Record<string, any> | undefined) => void;
    state: vue_demi.ComputedRef<Record<string, any>>;
    readonly store: ManagerData;
}> | undefined) => void;

export { CreateGlobalStoreArgs, CreateLocalStoreArgs, CreateSetupArgs, CreateStoreOptions, DropFunctionObject, EmptyObject, ManagerData, ObjectWithRef, OneAndAnyFunc, Plugin, PluginOptions, STORE_CONTEXT, STORE_LOCAL_CONTEXT, Setup, SetupContext, Store, StoreManager, StoreManagerItem, StoreTree, StoreTreeInfo, UnwrapNestedRefs, UseStore, UseStoreOptions, Vare, createInspect, createManager, createStore, createStoreDevTool, createTree, createTreeMultiple, createVare, createVareStore, defineStore, provideStoreManager, useLocalManager, useStore, useStoreManager };
