var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));

// src/store/index.ts
import { markRaw, provide } from "vue-demi";

// src/store/manager.ts
import { freeze } from "@winter-love/utils";
import { computed, inject, reactive } from "vue-demi";

// src/store/symbols.ts
var STORE_CONTEXT = Symbol("store");
var STORE_LOCAL_CONTEXT = Symbol("store-local");

// src/store/manager.ts
var useStoreManager = () => {
  var _a;
  return (_a = inject(STORE_CONTEXT)) != null ? _a : createManager();
};
var { assign } = Object;
var createManager = (info) => {
  const storeTree = reactive({});
  const state = computed(() => storeTree);
  const _info = freeze(info);
  const store = freeze({
    info: _info,
    tree: storeTree
  });
  const initState = {};
  const storePropsMap = /* @__PURE__ */ new Map();
  const set = (name, item) => {
    storeTree[name] = item.state;
  };
  const remove = (name) => {
    storeTree[name] = void 0;
    storePropsMap.delete(name);
  };
  const get = (name) => {
    const state2 = storeTree[name];
    if (!state2) {
      return;
    }
    return {
      state: state2
    };
  };
  const setInitState = (state2 = {}) => {
    assign(initState, state2);
  };
  return {
    get,
    initState,
    remove,
    set,
    setInitState,
    state,
    store
  };
};

// src/store/dev-tool.ts
import { setupDevtoolsPlugin } from "@vue/devtools-api";
import { drop, parseJson, toArray } from "@winter-love/utils";
import { watch } from "vue-demi";
var tagThemes = {
  "local-state": {
    backgroundColor: 7580670,
    label: "local state",
    textColor: 0
  },
  state: {
    backgroundColor: 394748,
    label: "state",
    textColor: 0
  },
  unknown: {
    backgroundColor: 16758528,
    label: "unknown",
    textColor: 0
  }
};
var createTree = (store, index = 0) => {
  const { info, tree } = store;
  return Object.keys(tree).map((key) => {
    var _a;
    const itemInfo = parseJson(key, key);
    const tag = typeof itemInfo === "string" ? [] : [
      {
        backgroundColor: 10066329,
        label: `for: ${itemInfo.componentName}`,
        textColor: 0
      },
      {
        backgroundColor: 10066329,
        label: itemInfo.uid,
        textColor: 0
      }
    ];
    return {
      id: JSON.stringify({
        index,
        key
      }),
      label: typeof itemInfo === "string" ? itemInfo : itemInfo.name,
      tags: [
        __spreadValues({}, tagThemes[(_a = info == null ? void 0 : info.kind) != null ? _a : "unknown"]),
        ...tag
      ]
    };
  });
};
var createTreeMultiple = (...stores) => {
  return stores.flatMap((store, index) => {
    return createTree(store, index);
  });
};
var createInspect = (name, state) => {
  const itemInfo = parseJson(name, name);
  return {
    state: [
      {
        editable: true,
        key: typeof itemInfo === "string" ? itemInfo : itemInfo.name,
        objectType: "reactive",
        value: state
      }
    ]
  };
};
var createStoreDevTool = (app, stores) => {
  const inspectId = "vare-inspect";
  const timeLineId = "vare-event";
  const label = "vare-store";
  const _stores = toArray(stores);
  setupDevtoolsPlugin({
    app,
    id: "vare-store",
    label: "vare-store",
    packageName: "vare"
  }, (api) => {
    api.addInspector({
      icon: "mediation",
      id: inspectId,
      label: `${label} Structure`
    });
    api.addTimelineLayer({
      color: 15764809,
      id: timeLineId,
      label: "Vare Changing"
    });
    api.on.getInspectorTree((payload) => {
      if (payload.app !== app || payload.inspectorId !== inspectId) {
        return;
      }
      payload.rootNodes = createTreeMultiple(..._stores);
    });
    api.on.getInspectorState((payload) => {
      if (payload.app !== app || payload.inspectorId !== inspectId) {
        return;
      }
      const name = payload.nodeId;
      const { key, index } = parseJson(name, { index: 0, key: name });
      const state = _stores[index].tree[key];
      if (state) {
        payload.state = createInspect(key, state);
      }
    });
    api.on.editInspectorState((payload) => {
      if (payload.app !== app || payload.inspectorId !== inspectId) {
        return;
      }
      const name = payload.nodeId;
      const { key, index } = parseJson(name, { index: 0, key: name });
      const state = _stores[index].tree[key];
      if (state) {
        const path = drop(1)(payload.path);
        const { value } = payload.state;
        payload.set(state, path, value);
      }
    });
    Object.keys(_stores).forEach((name) => {
      const store = _stores[name];
      watch(store.tree, () => {
        api.addTimelineEvent({
          event: {
            data: typeof store.tree === "object" ? __spreadValues({}, store.tree) : store.tree,
            time: Date.now()
          },
          layerId: timeLineId
        });
      });
    });
  });
};

// src/store/store.ts
import { freeze as freeze2 } from "@winter-love/utils";
import { getCurrentInstance, inject as inject2, onScopeDispose, reactive as reactive2, toRaw } from "vue-demi";

// src/store/create-uuid.ts
var createUuid = () => {
  let count = 0;
  return () => {
    count += 1;
    return count;
  };
};

// src/store/store.ts
var getUuid = createUuid();
var createSetup = (args) => {
  const { setup, storeManager, initState } = args;
  return {
    state: reactive2(setup(initState != null ? initState : {}, storeManager.state.value))
  };
};
var createLocalStore = (args) => {
  var _a, _b, _c;
  const { name, storeManager, setup } = args;
  const item = createSetup({
    initState: {},
    setup,
    storeManager
  });
  const instance = getCurrentInstance();
  const uid = instance ? instance.uid : getUuid();
  const componentName = (_c = (_b = (_a = instance == null ? void 0 : instance.ctx) == null ? void 0 : _a.$options) == null ? void 0 : _b.name) != null ? _c : "Undefined Name";
  const uuidName = process.env.NODE_ENV === "production" ? `${uid}` : JSON.stringify({
    componentName,
    name,
    uid
  });
  const localStoreManager = useLocalManager();
  if (localStoreManager) {
    localStoreManager.set(uuidName, item);
  }
  onScopeDispose(() => {
    localStoreManager == null ? void 0 : localStoreManager.remove(uuidName);
  });
  return item.state;
};
var createGlobalStore = (args) => {
  var _a;
  const { name, options, storeManager, setup } = args;
  const { reset } = options;
  const savedState = storeManager.get(name);
  if (savedState && !reset) {
    return savedState.state;
  }
  const initState = toRaw((_a = storeManager.initState[name]) != null ? _a : {});
  const newState = createSetup({
    initState,
    setup,
    storeManager
  });
  storeManager.set(name, newState);
  return newState.state;
};
var useLocalManager = () => {
  return inject2(STORE_LOCAL_CONTEXT);
};
var getOptions = (arg1, arg2) => {
  if (typeof arg1 === "string" && typeof arg2 !== "undefined") {
    return {
      name: arg1,
      setup: arg2
    };
  }
  if (typeof arg1 === "object" && typeof arg2 === "undefined") {
    return arg1;
  }
  throw new Error("params error");
};
function defineStore(arg1, arg2) {
  const {
    name,
    setup,
    local
  } = getOptions(arg1, arg2);
  const hook = (options = {}) => {
    const { local: innerLocal = local } = options;
    const storeManager = useStoreManager();
    if (innerLocal) {
      return createLocalStore({ name, setup, storeManager });
    }
    return createGlobalStore({
      name,
      options,
      setup,
      storeManager
    });
  };
  return freeze2(Object.assign(hook, {
    local: (options = {}) => hook(__spreadProps(__spreadValues({}, options), { local: true }))
  }));
}
function createStore(arg1, arg2) {
  return defineStore(arg1, arg2);
}
function useStore(arg1, arg2) {
  return createStore(arg1, arg2)();
}

// src/store/index.ts
var createVare = () => {
  const manager = createManager({
    kind: "state"
  });
  const localManager = createManager({
    kind: "local-state"
  });
  return markRaw({
    install: (app, options = {}) => {
      const { plugins = [], initState } = options;
      manager.setInitState(initState);
      plugins.forEach((plugin) => {
        plugin(manager.state, manager.store.info);
      });
      app.provide(STORE_CONTEXT, manager);
      app.provide(STORE_LOCAL_CONTEXT, localManager);
    },
    localManager,
    manager,
    state: manager.state
  });
};
var createVareStore = createVare;
var provideStoreManager = (manager, localManager) => {
  provide(STORE_CONTEXT, manager != null ? manager : createManager());
  if (process.env.NODE_ENV !== "production") {
    provide(STORE_LOCAL_CONTEXT, localManager);
  }
};
export {
  STORE_CONTEXT,
  STORE_LOCAL_CONTEXT,
  createInspect,
  createManager,
  createStore,
  createStoreDevTool,
  createTree,
  createTreeMultiple,
  createVare,
  createVareStore,
  defineStore,
  provideStoreManager,
  useLocalManager,
  useStore,
  useStoreManager
};
//# sourceMappingURL=index.mjs.map